/*
 * ----------------------------------------------
 *
 * This software belongs to the zzzutils package.
 *
 * Copyright (c) 2003 Fabian "zzznowman" Pietsch.
 * All rights reserved.
 *
 * ----------------------------------------------
 */

/*
 * zbtree.h -- provide an unbalanced b-tree implementation for zzzutils
 *
 **********************************************************************
 *
 * brief ChangeLog:
 *   2003-08-06 to -08 first implementation inside of zzzupdate-xmms.c
 *   2003-08-08 abstraction as zbtree.{c,h}
 *
 **********************************************************************
 *
 *
 */


/* handle multiple inclusions */
#ifndef __ZBTREE_H
#define __ZBTREE_H

/* zbtree_enum.stack incrementation step size
 * zbtree_next() will enlarge/shrink its enumeration stack
 * by this amount of elements at a time
 */
#if 0
#define ZBTREE_SINC 4
#endif
#define ZBTREE_SINC 256


/* zbtree base structure */
struct zbtree
{
	char *v;		/* value		*/
	struct zbtree *n_le;	/* next, less or equal	*/
	struct zbtree *n_gt;	/* next, greater	*/
};

/* zbtree_e*() enumeration status preservation */
struct zbtree_enum
{
	int stack_depth;	/* elements currently on stack	*/
	int stack_size;		/* elements fitting on stack	*/
	struct zbtree **stack;	/* the element stack itself	*/
	struct zbtree *root;	/* zbtree's root for this enum	*/
};

#if defined(ZBTREE_EDUMP)
/* zbtree_edump -- zbtree_enum corruption tracking */
void inline zbtree_edump(struct zbtree_enum *e)
{
	int i;
	
	zwarn("zbtree_edump(): ===begin=== for enum at %p", e);
	zwarn("zbtree_edump(): root == %p", e->root);
	zwarn("zbtree_edump(): stack_depth == %i, stack_size == %i", e->stack_depth, e->stack_size);

	/* check for general stack validness */
	if (e->stack_depth > e->stack_size)
		zerror("zbtree_edump(): !!! stack_depth > stack_size !!!");
	else
	{
		zwarn("zbtree_edump(): stack dump:");
		/* loop through e's stack */
		for (i = 0; i < e->stack_depth; i++)
		{
			zwarn("zbtree_edump(): e->stack[%i] == %p", i, e->stack[i]);
		}
	}
	
	zwarn("zbtree_edump(): === end === for enum at %p", e);
}
#endif


/*
 * prototypes for zbtree.c
 *************************
 *
 */

/*
 * general zbtree functions
 **********************************************************
 * zbtree_add() -- add a new element to a struct zbtree chain
 * zbtree_free() -- free an entire struct zbtree chain
 */
struct zbtree *zbtree_add(struct zbtree **bt, char *v);
void zbtree_free(struct zbtree **bt, const int freev);

/*
 * zbtree_e*() -- zbtree enumeration functions
 ***********************************************************************
 * zbtree_estart() has to be called first with the zbtree's root element
 * zbtree_enext() is called then to sequentially get the tree's elements
 * zbtree_efirst() is a wrapper around zbtree_enext():
 *     it resets the enumeration & returns the zbtree's first element
 * zbtree_ereset() resets the enumeration, so ending and starting again
 *     the same enumeration becomes unneccessarry
 * zbtree_eend() has to be called at the end of every enumeration, espe-
 *     cially if only partial -- it frees the data structures used
 */
struct zbtree_enum *zbtree_estart(struct zbtree *bt);
struct zbtree_enum *zbtree_eend(struct zbtree_enum *e);
struct zbtree *zbtree_efirst(struct zbtree_enum *e);
struct zbtree *zbtree_enext(struct zbtree_enum *e);
int zbtree_ereset(struct zbtree_enum *e);

#endif /* __ZTBTREE_H */
