/*
 * ----------------------------------------------------------------------------
 *
 * ztdb-my.c
 * Copyright (C) 2003 Fabian "zzznowman" Pietsch
 *
 * ----------------------------------------------------------------------------
 *
 */

/*
 * zzztel MySQL database backend module
 ******************************************************************************
 * MySQL backend for ztdb abstraction interface
 *
 */

#include "zzzutils.h"
#if 0
#include "zzztel.h"
#include "ztdb.h"
#endif
#include "ztdb-my.h"

#if 0
#include <stdlib.h>
#include <stdio.h>
#endif
#include <string.h>
#include <mysql.h>


/* MySQL client library data structures */
static MYSQL mysql;
static MYSQL_RES *mysqlres;


/* initialize MySQL environment */
int ztdb_mysql_init()
{
	int i;
	char *password;

	/* initialize MySQL object */
	if (mysql_init(&mysql) == NULL)
	{
		zerror("mysql_init(): %s", mysql_error(&mysql));
		return(-1);
	}
	
	/* read in my.cnf */
	mysql_options(&mysql, MYSQL_READ_DEFAULT_GROUP, ZTDB_GROUP);

	/* input MySQL password */
	if ((password = get_tty_password(NULL)) && (strlen(password) == 0))
	{
		/* try without a password / using my.cnf */
		password = NULL;
	}

	/* connect to MySQL server */
	if (mysql_real_connect(&mysql, ZTDB_HOST, ZTDB_USER, password,
	      ZTDB_DB, 0, NULL, 0) == NULL)
	{
		zerror("mysql_real_connect(): %s", mysql_error(&mysql));
		return(-1);
	}
	
	/* clear password */
	if (password)
		for (i=0; i<strlen(password); i++)
			password[i] = '\0';

	/* success */
	return(0);
}

/* deinitialize MySQL environment */
int ztdb_mysql_finish()
{
	/* close connection to MySQL server */
	mysql_close(&mysql);
	return(0);
}

/* do an SQL query without result set */
int ztdb_mysql_query_do(const char *query)
{
#if defined(DEBUG) && DEBUG
	zwarn("ztdb_mysql_query_do(\"%s\")", query);
#endif
	/* execute query */
	if (mysql_real_query(&mysql, query, strlen(query)))
	{
		zerror("mysql_real_query(): %s", mysql_error(&mysql));
		return(-1);
	}

	/* return number of affected rows */
	return((int)mysql_affected_rows(&mysql));
}

/* do an SQL query with result set */
int ztdb_mysql_query_res_init(const char *query)
{
	/* query MySQL */
	if (mysql_real_query(&mysql, query, strlen(query)))
	{
		zerror("mysql_real_query(): %s", mysql_error(&mysql));
		return(-1);
	}

	/* prepare result retrieval */
	if ((mysqlres = mysql_store_result(&mysql)) == NULL)
	{
		zerror("mysql_store_result(): %s", mysql_error(&mysql));
		return(-1);
	}

	/* success */
	return((int)mysql_num_rows(mysqlres));
}

/* fetch next telephone number from result set */
const char *const *ztdb_mysql_query_res_fetch()
{
	MYSQL_ROW row;

	/* fetch a row of raw data */
	if ((row = mysql_fetch_row(mysqlres)) == NULL)
	{
		/* zerror("can't fetch row: %s", mysql_error(&mysql)); */
		ztdb_mysql_query_res_finish();
		return(NULL);
	}

	/* return row as argv-like (char **) */
	return((const char *const *)row);
}

/* finish an SQL query with result set */
int ztdb_mysql_query_res_finish()
{
	/* free the query's result set */
	mysql_free_result(mysqlres);

	/* success */
	return(0);
}


/*
 * ~EOF
 */



#if 0
/* delete audio file from database */
static int ztdb_mysql_del(const char *name) {
	int name_len = strlen(name), query_len;
	char *name_esc, *query;
	my_ulonglong rows;

	/* allocate len*2 + 1 chars for mysql_real_escape_string */
	if ((name_esc = malloc(sizeof(char) * (name_len * 2 + 1))) == NULL) {
		zperror("ztdb_mysql_del(): malloc()");
		return(-1);
	}

	/* escape name prior to using it in MySQL query
	 * according to MySQL documentation, there are no possible errors...
	 */
	mysql_real_escape_string(&mysql, name_esc, name, name_len);

	/* get size of buffer needed for query */
	if ((query_len = snprintf(NULL, 0, ZTDB_SQL_DEL, name_esc)) < 1) {
		/* this snprintf isn't friendly enough... => estimate roughly */
		query_len = 1000;
	}

	/* allocate buffer for query */
	if ((query = malloc(sizeof(char) * (query_len + 1))) == NULL) {
		zperror("ztdb_mysql_del(): malloc()");
		free(name_esc);
		return(-1);
	}

	/* prepare query string */
	snprintf(query, query_len + 1, ZTDB_SQL_DEL, name_esc);
	free(name_esc);

	/* execute query */
	if (mysql_real_query(&mysql, query, strlen(query))) {
		zerror("ztdb_mysql_del(): mysql_real_query(): %s", mysql_error(&mysql));
		free(query);
		return(-1);
	}
	free(query);

	/* check result */
	if ((rows = mysql_affected_rows(&mysql)) != 1) {
		zerror("ztdb_mysql_del(): mysql_affected_rows() didn't return 1 but %d", rows);
		return(-1);
	}

	/* success */
	return(0);
}

static int ztdb_mysql_put(const char *name) {
	int name_len = strlen(name), query_len;
	char *name_esc, *query;
	my_ulonglong rows;

	/* allocate len*2 + 1 chars for mysql_real_escape_string */
	if ((name_esc = malloc(sizeof(char) * (name_len * 2 + 1))) == NULL) {
		zperror("ztdb_mysql_put(): malloc()");
		return(-1);
	}

	/* escape name prior to using it in MySQL query
	 * according to MySQL documentation, there are no possible errors...
	 */
	mysql_real_escape_string(&mysql, name_esc, name, name_len);

	/* get size of buffer needed for query */
	if ((query_len = snprintf(NULL, 0, ZTDB_SQL_PUT, name_esc)) < 1) {
		/* this snprintf isn't friendly enough... => estimate roughly */
		query_len = 1000;
	}

	/* allocate buffer for query */
	if ((query = malloc(sizeof(char) * (query_len + 1))) == NULL) {
		zperror("ztdb_mysql_put(): malloc()");
		free(name_esc);
		return(-1);
	}

	/* prepare query string */
	snprintf(query, query_len + 1, ZTDB_SQL_PUT, name_esc);
	free(name_esc);

	/* execute query */
	if (mysql_real_query(&mysql, query, strlen(query))) {
		zerror("ztdb_mysql_put(): mysql_real_query(): %s", mysql_error(&mysql));
		free(query);
		return(-1);
	}
	free(query);

	/* check result */
	if ((rows = mysql_affected_rows(&mysql)) != 1) {
		zerror("ztdb_mysql_put(): mysql_affected_rows() didn't return 1 but %d", rows);
		return(-1);
	}

	/* success */
	return(0);
}
#endif
