/*
 * ----------------------------------------------------------------------------
 *
 * ztdb.c
 * Copyright (C) 2003, 2004  Fabian Pietsch
 *
 * ----------------------------------------------------------------------------
 *
 */

/*
 * zzztel database module
 ******************************************************************************
 * database abstraction interface for zzztel
 *
 */

#include "zttn.h"
#include "ztlist.h"
#include "ztdb.h"
#include "ztdb-my.h"
#include "zzzutils.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>


/* SQL statement prototypes */
#define ZTDB_SQL_TN        "SELECT pre_int, pre_local, line, comment FROM telnums %s"
#define ZTDB_SQL_TNDEL     "DELETE FROM telnums %s"
#define ZTDB_SQL_TNINS     "INSERT INTO telnums (pre_int, pre_local, line, comment) VALUES %s"
#define ZTDB_SQL_BIT_LIMIT " LIMIT %d"


/* prototypes for private functions */
static char *ztdb_tnstow(const ZTSearch *search);


/* initialize database environment: connect to MySQL server */
int ztdb_init()
{
	/* initialize MySQL environment */
	return ztdb_mysql_init();
}

/* deinitialize database environment */
int ztdb_finish()
{
	/* deinitialize MySQL environment */
	return ztdb_mysql_finish();
}

/* search for telephone number */
int ztdb_tnquery(const ZTSearch *search)
{
	char *query, *where;
	int query_len, rows;

	/* transform search to SQL WHERE clause */
	if ((where = ztdb_tnstow(search)) == NULL)
	{
		zerror("can't transform search to SQL WHERE clause");
		return -1;
	}

	/* compute size of query string & allocate memory */
	if ((query_len = snprintf(NULL, 0, ZTDB_SQL_TN, where)) < 1)
		query_len = 1000;	/* poor snprintf() implementation */
	if ((query = malloc(sizeof(char) * (query_len + 1))) == NULL)
	{
		zperror("malloc()");
		return -1;
	}

	/* put query string together */
	snprintf(query, query_len + 1, ZTDB_SQL_TN, where);
	where = realloc(where, 0);

	/* execute query */
	rows = ztdb_mysql_query_res_init(query);
	query = realloc(query, 0);
	if (rows < 0)
	{
		zerror("can't query database for telephone numbers");
		return -1;
	}
	return rows;
}

/* get next telephone number found in search */
ZTTN *ztdb_tnget()
{
	const char *const *row;
	ZTTN *tn;

	/* fetch row from database */
	if ((row = ztdb_mysql_query_res_fetch()) == NULL)
		return NULL;

	/* allocate memory for struct zttelnum */
	if ((tn = malloc(sizeof(ZTTN))) == NULL)
	{
		zperror("malloc()");
		return NULL;
	}

	/* copy raw db data to struct zttelnum */
	tn->pre_int   = strdup(row[0]);
	tn->pre_local = strdup(row[1]);
	tn->line      = strdup(row[2]);
	tn->comment   = strdup(row[3]);

	/* return fetched telephone number */
	return tn;
}

/* get all telephone numbers found in search */
ZTList *ztdb_tngetall()
{
	ZTList *list;
	ZTTN   *tn;

	/* create telephone number list */
	if ((list = ztlist_new()) == NULL)
		return NULL;

	/* get all telephone numbers */
	while ((tn = ztdb_tnget()))
	{
		/* add telephone number to list */
		if ((ztlist_add(list, tn)) == NULL)
			goto err_free;
	}

	/* return filled list */
	return list;

err_free:
	/* free the list allocated so far and indicate error */
	ztlist_free(list);
	return NULL;
}

/* insert telephone number into database */
int ztdb_tnins(const ZTTN *tn)
{
	char *query, *values;
	int query_len, values_len, rows;

#define ZTDB_DEST "('%s', '%s', '%s', '%s')"
	/* put together values string */
	values_len = strlen(ZTDB_DEST) - 2*4
	             + strlen(tn->pre_int)
	             + strlen(tn->pre_local)
	             + strlen(tn->line)
	             + strlen(tn->comment);
	if ((values = malloc(sizeof(char) * (values_len + 1))) == NULL)
	{
		zperror("malloc()");
		return -1;
	}
	snprintf(values, values_len + 1, ZTDB_DEST,
	         tn->pre_int, tn->pre_local,
	         tn->line, tn->comment);
#undef ZTDB_DEST

	/* compute size of query string & allocate memory */
	if ((query_len = snprintf(NULL, 0, ZTDB_SQL_TNINS, values)) < 1)
		query_len = 1000;	/* poor snprintf() implementation */
	if ((query = malloc(sizeof(char) * (query_len + 1))) == NULL)
	{
		zperror("malloc()");
		return -1;
	}

	/* put together query string */
	snprintf(query, query_len + 1, ZTDB_SQL_TNINS, values);
	values = realloc(values, 0);

	/* execute query */
	rows = ztdb_mysql_query_do(query);
	query = realloc(query, 0);
	if (rows < 0)
	{
		zerror("can't insert telephone number into database");
		return -1;
	}
	return rows;
}

/* delete a single telephone number from database */
int ztdb_tndel(const ZTTN *tn)
{
	ZTSearch search;
	int ret;

	assert(tn != NULL);

	/* set up search object */
	search.tn = (ZTTN *) tn;
	search.fields = ZTS_ALL;
	search.limit = 1;

	/* perform deletion */
	ret = ztdb_tndelall(&search);

	/* check the result (number of rows affected or error if negative) */
	if (ret >= 1)
		/* indicate success */
		return 0;

	/* nothing deleted or error */
	return (ret < 0 ? ret : -1);
}

/* delete telephone numbers from database */
int ztdb_tndelall(const ZTSearch *search)
{
	char *query, *where;
	int query_len, rows;

	/* transform search to SQL WHERE clause */
	if ((where = ztdb_tnstow(search)) == NULL)
	{
		zerror("can't transform search to SQL WHERE clause");
		return -1;
	}

	/* compute size of query string & allocate memory */
	if ((query_len = snprintf(NULL, 0, ZTDB_SQL_TNDEL, where)) < 1)
		query_len = 1000;	/* poor snprintf() implementation */
	if ((query = malloc(sizeof(char) * (query_len + 1))) == NULL)
	{
		zperror("malloc()");
		free(where);
		return -1;
	}

	/* put query string together */
	snprintf(query, query_len + 1, ZTDB_SQL_TNDEL, where);
	where = realloc(where, 0);

	/* execute query */
	rows = ztdb_mysql_query_do(query);
	query = realloc(query, 0);
	if (rows < 0)
	{
		zerror("can't delete telephone numbers from database");
		return -1;
	}

	return rows;
}

/* define some string handling macros */
#define zt_raf(p, size) do { \
	void *pnew; \
	if ((pnew = realloc(p, size)) == NULL) { \
		zperror("realloc()"); \
		free(p); \
		return NULL; \
	} \
	p = pnew; \
} while (0)
#define zts_strcat(src) do { \
	zt_raf(where, where_len + strlen(src) + 1); \
	where[where_len] = '\0'; \
	strcat(where, src); \
	where_len = strlen(where); \
} while (0)
#define zts_append(field, value) do { \
	if (items++) zts_strcat(" AND "); \
	else zts_strcat("WHERE "); \
	zts_strcat(field " = '"); \
	zts_strcat(value); \
	zts_strcat("'"); \
} while (0)

/* convert search to SQL WHERE clause */
static char *ztdb_tnstow(const ZTSearch *search)
{
	char *where = NULL;
	int where_len = 0, items = 0;

	/* specific search? */
	if (search)
	{
		if (ZTS_HAS_PRE_INT(search->fields))
			zts_append("pre_int", search->tn->pre_int);
		if (ZTS_HAS_PRE_LOCAL(search->fields))
			zts_append("pre_local", search->tn->pre_local);
		if (ZTS_HAS_LINE(search->fields))
			zts_append("line", search->tn->line);
		if (ZTS_HAS_COMMENT(search->fields))
			zts_append("comment", search->tn->comment);
	}

	/* is where still NULL? set it to empty string, then */
	if (!where)
	{
		if ((where = malloc(sizeof(char))) == NULL)
		{
			zperror("malloc()");
			return NULL;
		}
		where[0] = '\0';
	}

	/* append limit clause, if desired */
	if (search && (search->limit > 0))
	{
		char *limit, *where_new;
		int   limit_len;

		/* compute size for LIMIT clause buffer */
		if ((limit_len = snprintf(NULL, 0, ZTDB_SQL_BIT_LIMIT, search->limit)) < 1)
			/* poor snprintf() implementation */
			limit_len = 10;

		/* allocate memory for buffer */
		if ((limit = malloc((sizeof(char)) * (limit_len + 1))) == NULL)
		{
			zperror("malloc()");
			free(where);
			return NULL;
		}

		/* build LIMIT clause */
		snprintf(limit, limit_len + 1, ZTDB_SQL_BIT_LIMIT, search->limit);

		/* resize WHERE clause buffer */
		if ((where_new = realloc(where, where_len + limit_len + 1)) == NULL)
		{
			zperror("realloc()");
			free(limit);
			free(where);
			return NULL;
		}
		where = where_new;

		/* append LIMIT clause to WHERE clause */
		strncat(where, limit, limit_len);
		free(limit);
		where_len += limit_len;
	}

	/* return SQL WHERE clause */
	return where;
}

#undef zts_append
#undef zts_strcat
#undef zt_raf
