/*
 * ----------------------------------------------------------------------------
 *
 * ztlist.c -- list of telephone numbers for zzztel
 * Copyright (C) 2004  Fabian Pietsch
 *
 * ----------------------------------------------------------------------------
 *
 */

/*
 * zzztel telephone numbers list
 ******************************************************************************
 * list of telephone numbers for zzztel
 *
 */

#include "ztlist.h"
#include "zzzutils.h"
#include <stdlib.h>
#include <assert.h>

/* create new telephone number list element */
ZTListEl *ztlistel_new(ZTTN *tn)
{
	ZTListEl *element;

	/* allocate memory for list element */
	if ((element = malloc(sizeof(ZTListEl))) == NULL)
	{
		zperror("malloc()");
		return NULL;
	}

	/* set up list element */
	if ((element->tn = zttn_new(tn)) == NULL)
	{
		free(element);
		return NULL;
	}
	element->next = NULL;

	/* return list element */
	return element;
}

/* deallocate telephone number list element */
void ztlistel_free(ZTListEl *element)
{
	assert(element != NULL);

	/* free stored telephone number */
	zttn_free(element->tn);

	/* free next element in linked list */
	if (element->next)
	{
		ztlistel_free(element->next);
	}

	/* free list element itself */
	free(element);
}

/* create new telephone number list */
ZTList *ztlist_new()
{
	ZTList *list;

	/* allocate memory for list */
	if ((list = malloc(sizeof(ZTList))) == NULL)
	{
		zperror("malloc()");
		return NULL;
	}

	/* set up list */
	list->count = 0;
	list->first = NULL;
	list->end = &list->first;

	/* return list */
	return list;
}

/* add new item to telephone number list */
ZTListEl *ztlist_add(ZTList *list, ZTTN *tn)
{
	ZTListEl *element;

	/* create new list element */
	if ((element = ztlistel_new(tn)) == NULL)
		return NULL;

	/* link new element into list */
	*list->end = element;
	list->end = &element->next;
	list->count++;

	/* return new list element */
	return element;
}

/* delete item from telephone number list */
int ztlist_del(ZTList *list, ZTListEl *element)
{
	ZTListEl **hook;

	assert((list != NULL) && (element != NULL));

	hook = &list->first;

	/* find element in linked list */
	while (*hook != element)
	{
		if (*hook == NULL)
			/* end of list reached; element not found */
			return 1;

		hook = &(*hook)->next;
	}

	/* unlink element from list */
	*hook = element->next;
	list->count--;

	/* adjust list's end pointer */
	if (element->next == NULL)
	{
		assert(list->end == &element->next);
		list->end = hook;
	}

	/* free list element */
	element->next = NULL;
	ztlistel_free(element);

	/* success */
	return 0;
}

/* convert telephone number list index to list element pointer */
ZTListEl *ztlist_idx_to_ptr(ZTList *list, int index)
{
	ZTListEl *i_element;
	int       i_index;

	assert(list != NULL);

	/* iterate through list */
	i_element = list->first;
	i_index = 0;
	while (i_element)
	{
		/* is this the element requested? */
		if (i_index == index)
			return i_element;

		/* advance to next element in list */
		i_element = i_element->next;
		i_index++;
	}

	/* index out of range; indicate error */
	return NULL;
}

/* convert telephone number list element pointer to list index */
int ztlist_ptr_to_idx(ZTList *list, ZTListEl *element)
{
	ZTListEl *i_element;
	int       i_index;

	assert(list != NULL);

	/* iterate through list */
	i_element = list->first;
	i_index = 0;
	while (i_element)
	{
		/* does this element have the index requested? */
		if (i_element == element)
			return i_index;

		/* advance to next element in list */
		i_element = i_element->next;
		i_index++;
	}

	/* element not in list; indicate error */
	return -1;
}

/* deallocate telephone number list and contained data */
void ztlist_free(ZTList *list)
{
	/* free linked list of list elements */
	ztlistel_free(list->first);

	/* free list itself */
	free(list);
}
